/* *****************************************************************
 *
 *  $Id: f0m_lib.c,v 1.4 2002/09/27 12:43:01 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/09/16 19:05:07 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
  *  Description: Implementation of the services provided by FIDDLE-1
 *
 *		f0m_initialize ()		-- Initialize FIDDLE-1
 *		f0m_terminate ()			-- Terminate FIDDLE-1
 *		f0m_tids ()				-- List existing task IDs
 *		f0m_kill ()				-- Kill a target process
 *		f0m_attach ()			-- Attach to a running process
 *		f0m_detach ()			-- Detach from a running process
 *		f0m_symbol_file ()		-- Specify whre to find program symbols
 *		f0m_file ()				-- Load program and symbols into memory
 *		f0m_run ()				-- Run a program
 *		f0m_step ()				-- Executite until the next instruction
 *		f0m_next ()				-- Executite until the next instruction
 *		f0m_continue ()			-- Continue the execution
 *		f0m_finish ()			-- Execute until returning from
 *								   current stack frame
 *		f0m_signal ()			-- Send a signal to a process
 *		f0m_break ()				-- Set a breakpoint
 *		f0m_delete ()			-- Delete a breakpoint
 *		f0m_set_variable ()		-- Change the contents of a variable
 *		f0m_call ()				-- Call a function in the current
 *								   context of the target process
 *		f0m_evaluate ()			-- Evaluate an expression in the current
 *								   context of the target process
 *		f0m_display ()			-- Evaluate and display  an expression 
 *								   every time the process execution stops
 *		f0m_undisplay ()			-- Undisplay an expression
 *		f0m_list ()				-- List the process source code
 *		f0m_info_locals ()		-- Get name and value of local variables
 *		f0m_info_args ()		-- Get name and value of function argument
 *		f0m_info_line ()			-- Get info about current line
 *		f0m_info_program ()		-- Get info about current program
 *		f0m_info_stack ()		-- Get info about current stack frames
 *		f0m_up ()				-- Go up in the stack frame list
 *		f0m_down ()				-- Go down in the stack frame list
 *		f0m_frame ()				-- Select a specific stack frame
 *		f0m_info_break ()		-- Get info about current breakpoint
 *		f0m_info_display ()		-- Get info about current display expressions
 *		f0m_thread ()			-- Select a thread
 *		f0m_info_threads ()		-- Get info about the existing threads
 *		f0m_tty ()				-- Set a TTY for future IO
 *		f0m_sendto ()			-- Send a command directly to the debugger
 *		f0m_tkin3 ()			-- Send a 'tkin_t' directly to the debugger
 *
 * -----------------------------------------------------------------
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config.h"

#include "f0m_queues.h"
#include "f0s_tkout.h"
#include "f0s_out_codes.h"
#include "f0m_tkin.h"
#include "f0m_back_end.h"
#include "f0m_lib.h"


/* #################################################################
 *		#DEFINES, TYPES and GLOBAL (PRIVATE) VARIABLES
 * ################################################################# */

#define DELETE_AFTER 1

/* #################################################################
 *		PRIVATE functions
 * ################################################################# */


/* #################################################################
 *		PUBLIC functions
 * ################################################################# */


 
/* -----------------------------------------------------------------
 * Function:    f0m_initialize()
 *
 * Description: Initialize all the necessary data structures
 *
 * Returns:     The status
 */
PUBLIC int f0m_initialize (void)
{
#ifdef EBUG
	puts ("f0m_initialize()");
#endif

	/* Initialize FIDDLE-0 */
	f0s_initialize ();

	/* Initialize Queues */
	f0m_queues_init ();
	
	/* Start system thread */
	f0m_be_initialize ();

	return 0;
}


/* -----------------------------------------------------------------
 * Function:    f0m_terminate()
 *
 * Description: Terminate the debugging in this node.  Detach from all 
 *				processes and kil all CLDs.
 *
 * Returns:     The status
 */
PUBLIC int f0m_terminate (void)
{
#ifdef EBUG
	puts ("f0m_terminate()");
#endif
	/* Terminate system thread */
	f0m_be_terminate ();
	
	/* Terminate Queues */
	f0m_queues_terminate ();

	/* Terminate FIDDLE-0 */
	return f0s_terminate ();
}


/* -----------------------------------------------------------------
 * Function:    f0m_tids()
 *
 * Description: Get a list of all existing process TIDs.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_tids (const char *options)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_tids()");
#endif
	req = f0m_tkin_new (NO_TID, C_TIDS, options);
	return f0m_tkin3 (NO_TID, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_kill()
 *
 * Description: Kill a process being debugger but let the CLD
 *				    available for a next usage
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_kill (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_kill()");
#endif
	req = f0m_tkin_new (tid, C_KILL);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_add_debugger()
 *
 * Description: Add a 'ready to be controlled' debugger to Fiddle
 *				'dbg_pid' should contain the PID of the debugger
 *				'tp_pid' should contain the PID of the target process
 *						 or -1 if process is not loaded or not running yet
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_add_debugger (int dbg_pid, int tp_pid,
								  const char *cmd_channel,
								  const char *rep_channel)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_add_debugger()");
#endif
	req = f0m_tkin_new (NO_TID, C_ADD_DEBUGGER, NULL, dbg_pid, tp_pid,
						cmd_channel, rep_channel);
	return f0m_tkin3 (NO_TID, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0m_attach (int tp_pid)
 *
 * Description: Find an unsused component-level debugger or start one
 *				if none was found.  Then attach the debugger to the
 *				process 'tp_pid'.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_attach (int tp_pid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_attach()");
#endif
	req = f0m_tkin_new (NO_TID, C_ATTACH, tp_pid, NULL);
return f0m_tkin3 (NO_TID, req, DELETE_AFTER);}


/* -----------------------------------------------------------------
 * Function:    f0m_detach()
 *
 * Description: Detach a component-level debugger from the process
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_detach (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_detach()");
#endif
	req = f0m_tkin_new (tid, C_DETACH);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_symbol_file()
 *
 * Description: Load the symbols
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_symbol_file (int tid, const char *file)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_symbol_file()");
#endif
	req = f0m_tkin_new (tid, C_SYMBOL_FILE, file);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_file()
 *
 * Description: Load a process image into memory
 *				'tid == -1' load a new process image
 *				'tid != -1' reload a process image
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_file (int tid, const char *file)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_file()");
#endif
	if (tid == -1)
		tid = NO_TID;
	req = f0m_tkin_new (tid, C_FILE, file, NULL);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_run()
 *
 * Description: Start the execution of a process
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_run (int tid, const char *args)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_run()");
#endif
	req = f0m_tkin_new (tid, C_RUN, args);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_step()
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps into the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_step (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_step()");
#endif
	req = f0m_tkin_new (tid, C_STEP);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_next()
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps over the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_next (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_next()");
#endif
	req = f0m_tkin_new (tid, C_NEXT);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_continue()
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_continue (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_continue()");
#endif
	req = f0m_tkin_new (tid, C_CONTINUE);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_finish()
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_finish (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_finish()");
#endif
	req = f0m_tkin_new (tid, C_FINISH);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_signal()
 *
 * Description: Send a signal to a process.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_signal (int tid, int signum)
{
       int reqid;
       tkin_t *req;
       tkout_t *rep;

#ifdef EBUG
	puts ("f0m_signal()");
#endif
       req = f0m_tkin_new (tid, C_SIGNAL, tid, signum);
       assert (req != NULL);
       reqid = f0m_enqueue_request (NO_TID, req);
       assert (reqid >= 0);
       rep = f0m_dequeue_reply (reqid);
       return rep;
}


/* -----------------------------------------------------------------
 * Function:    f0m_break()
 *
 * Description: Set a breakpoint.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_break (int tid, const char *file, int line, const char *function)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_break()");
#endif
	req = f0m_tkin_new (tid, C_BREAK, file, line, function);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_delete()
 *
 * Description: Remove a breakpoint.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_delete (int tid, int bpid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_delete()");
#endif
	req = f0m_tkin_new (tid, C_DELETE, bpid);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_set_variable()
 *
 * Description: Set 'variable' to 'expression'
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_set_variable (int tid, const char *variable, const char *expression)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_set_variable()");
#endif
	req = f0m_tkin_new (tid, C_SET_VARIABLE, variable, expression);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_evaluate()
 *
 * Description: Evaluate an 'expression'
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_evaluate (int tid, const char *expression)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_evaluate()");
#endif
	req = f0m_tkin_new (tid, C_EVALUATE, expression);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_display()
 *
 * Description: Display 'expression' every time the program stops
 *				If 'expression == NULL' list list all expressions/values
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_display (int tid, const char *expression)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_display()");
#endif
	req = f0m_tkin_new (tid, C_DISPLAY, expression);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_undisplay()
 *
 * Description: Remove a display expression
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_undisplay (int tid, int dispid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_undisplay()");
#endif
	req = f0m_tkin_new (tid, C_UNDISPLAY, dispid);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkof0m_call()
 *
 * Description: Call a function and get it's return value
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_call (int tid, const char *expression)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_call()");
#endif
	req = f0m_tkin_new (tid, C_CALL, expression);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_list()
 *
 * Description: List a source file.  
 *              The arguments 'start_line' and 'function' are exclusive.
 *              The arguments 'n_lines' is optional.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_list (int tid, const char *file, int start_line, const char *function, int n_lines)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_list()");
#endif
	req = f0m_tkin_new (tid, C_LIST, file, start_line, function, n_lines);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkof0m_tty()
 *
 * Description: Set the I/O tty for the process being debugged
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_tty (int tid, const char *file)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_tty()");
#endif
	req = f0m_tkin_new (tid, C_TTY, file);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_locals()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_locals (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_locals()");
#endif
	req = f0m_tkin_new (tid, C_INFO_LOCALS);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_args()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_args (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_args()");
#endif
	req = f0m_tkin_new (tid, C_INFO_ARGS);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_info_line()
 *
 * Description: Get information about a source line.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_line (int tid, const char *file, int line, const char *function)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_line()");
#endif
	req = f0m_tkin_new (tid, C_INFO_LINE, file, line, function);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_program()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_program (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_program()");
#endif
	req = f0m_tkin_new (tid, C_INFO_PROGRAM, req);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_info_stack()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_stack (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_stack()");
#endif
	req = f0m_tkin_new (tid, C_INFO_STACK);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_up()
 *
 * Description: Go "up" on the stack
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_up (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_up()");
#endif
	req = f0m_tkin_new (tid, C_UP);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_down()
 *
 * Description: Go "down" on the stack
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_down (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_down()");
#endif
	req = f0m_tkin_new (tid, C_DOWN);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_frame()
 *
 * Description: Select a specific stack frame
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_frame (int tid, int frame)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_frame()");
#endif
	req = f0m_tkin_new (tid, C_FRAME, frame);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_info_break()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_break (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_break()");
#endif
	req = f0m_tkin_new (tid, C_INFO_BREAK);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_info_display()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_display (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_display()");
#endif
	req = f0m_tkin_new (tid, C_INFO_DISPLAY);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_thread()
 *
 * Description: Switch the current thread.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_thread (int tid, int threadID)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_thread()");
#endif
	req = f0m_tkin_new (tid, C_THREAD, threadID);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_info_threads()
 *
 * Description: Get information about the existing threads.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_info_threads (int tid)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_info_threads()");
#endif
	req = f0m_tkin_new (tid, C_INFO_THREADS);
	return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_sendto()
 *
 * Description: Send a command string directly to the debugger.
 *				'tkout' contains the output (NOT PARSED).
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *f0m_sendto (int tid, const char *command)
{
	tkin_t *req;

#ifdef EBUG
	puts ("f0m_sendto()");
#endif
	req = f0m_tkin_new (tid, C_SENDTO, command);
    return f0m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3_send_request()
 *
 * Description: Send a command already packed in a 'tkin_t' strucure
 *				to process 'tid'.  Does not wait for the reply.
 *
 * Returns:     The request ID
 */
PUBLIC int
f0m_tkin3_send_request (int tid, tkin_t *tk)
{
	int reqid;					/* To temporarily hold the request ID */
#ifdef EBUG
	char *str;					/* Needed to covert a 'tkin_t' to text
								   if debugging */
#endif
	
	assert (tk != NULL);

#ifdef EBUG
	msg_pid_write ("%s::%s()::Queuing a TKIN", __FILE__, __FUNCTION__);
	str = f0m_tkin_2_str (tk);
	msg_pid_write ("+-----------------------------------------------------+");
	msg_pid_write ("|  tkin                                               |");
	msg_pid_write ("+-----------------------------------------------------+");
	msg_pid_write ("%s\n", str);
	free (str);
#endif
	
	reqid = f0m_enqueue_request (tid, tk);

#ifdef EBUG
	msg_pid_write ("%s::%s()::Request ID = %d", __FILE__, __FUNCTION__, reqid);
#endif

	/* Return the request ID */
	return reqid;
}


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3_get_reply()
 *
 * Description: Get the reply associated with a certain request
 *				ID. Should use (-1) as the request ID to get any reply.
 *
 * Returns:     The output token
 */
PUBLIC tkout_t *
f0m_tkin3_get_reply (int reqid)
{
	tkout_t *rep;				/* To temporarily hold the reply */
#ifdef EBUG
	char *str;					/* Needed to covert a 'tkout_t' to
								   text if debugging */
#endif
	
	if (reqid < -1)
		return f0s_tkout_new (E_RTS_INVALID_REQID);

	/* (reqid >= -1) */
	rep = f0m_dequeue_reply (reqid);

#ifdef EBUG
	msg_pid_write ("%s::%s()::Dequeuing a TKOUT", __FILE__, __FUNCTION__);
	str = f0m_tkout_2_str (rep);
	msg_pid_write ("+-----------------------------------------------------+");
	msg_pid_write ("|  tkout                                              |");
	msg_pid_write ("+-----------------------------------------------------+");
	msg_pid_write ("%s\n", str);
	free (str);
#endif
	return rep;
}


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3()
 *
 * Description: Send a command already packed into a 'tkin_t' strucure
 *				to process 'tid'.
 *
 * Returns:     The 'Output Token'.
 */
PUBLIC tkout_t *
f0m_tkin3 (int tid, tkin_t *tk, int delete)
{
	int reqid = f0m_tkin3_send_request (tid, tk);
	tkout_t *rep =  f0m_tkin3_get_reply (reqid);

	/* Delete the request token? */
	if (delete)
		f0m_tkin_delete (tk);

	return rep;
}
