/* *****************************************************************
 *
 *  $Id: f0m_lib.h,v 1.4 2002/08/13 15:53:11 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/13 16:50:57 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description: Prototypes of the services provided by FIDDLE-1
 *
 *		f0m_initialize ()		-- Initialize FIDDLE-1
 *		f0m_terminate ()			-- Terminate FIDDLE-1
 *		f0m_tids ()				-- List existing task IDs
 *		f0m_kill ()				-- Kill a target process
 *		f0m_attach ()			-- Attach to a running process
 *		f0m_detach ()			-- Detach from a running process
 *		f0m_symbol_file ()		-- Specify whre to find program symbols
 *		f0m_file ()				-- Load program and symbols into memory
 *		f0m_run ()				-- Run a program
 *		f0m_step ()				-- Executite until the next instruction
 *		f0m_next ()				-- Executite until the next instruction
 *		f0m_continue ()			-- Continue the execution
 *		f0m_finish ()			-- Execute until returning from
 *								   current stack frame
 *		f0m_signal ()			-- Send a signal to a process
 *		f0m_break ()				-- Set a breakpoint
 *		f0m_delete ()			-- Delete a breakpoint
 *		f0m_set_variable ()		-- Change the contents of a variable
 *		f0m_call ()				-- Call a function in the current
 *								   context of the target process
 *		f0m_evaluate ()			-- Evaluate an expression in the current
 *								   context of the target process
 *		f0m_display ()			-- Evaluate and display  an expression 
 *								   every time the process execution stops
 *		f0m_undisplay ()			-- Undisplay an expression
 *		f0m_list ()				-- List the process source code
 *		f0m_info_locals ()		-- Get name and value of local variables
 *		f0m_info_args ()		-- Get name and value of function argument
 *		f0m_info_line ()		-- Get info about current line
 *		f0m_info_program ()		-- Get info about current program
 *		f0m_info_stack ()		-- Get info about current stack frames
 *		f0m_up ()				-- Go up in the stack frame list
 *		f0m_down ()				-- Go down in the stack frame list
 *		f0m_frame ()				-- Select a specific stack frame
 *		f0m_info_break ()		-- Get info about current breakpoint
 *		f0m_info_display ()		-- Get info about current display expressions
 *		f0m_thread ()			-- Select a thread
 *		f0m_info_threads ()		-- Get info about the existing threads
 *		f0m_tty ()				-- Set a TTY for future IO
 *		f0m_sendto ()			-- Send a command directly to the debugger
 *		f0m_tkin ()				-- Send a "tkin_t" directly to the debugger
 *
 * -----------------------------------------------------------------
 */
#ifndef __F1_LIB_H
#define __F1_LIB_H


#include "glob.h"

#include "f0s_lib.h"
#include "f0m_tkin.h"


__BEGIN_DECLS

#define F0M_SERVER_HOST			"localhost"
#define F0M_SERVER_SERVICE_NUMBER	"9898"

 
/* -----------------------------------------------------------------
 * Function:    f0m_initialize()
 *
 * Description: Initialize all the necessary data structures
 *
 * Returns:     The status
 */
int f0m_initialize (void);


/* -----------------------------------------------------------------
 * Function:    f0m_terminate()
 *
 * Description: Terminate the debugging in this node.  Detach from all 
 *				processes and kil all CLDs.
 *
 * Returns:     The status
 */
int f0m_terminate (void);


/* -----------------------------------------------------------------
 * Function:    f0m_tids()
 *
 * Description: Get a list of all existing process TIDs.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_tids (const char *options);


/* -----------------------------------------------------------------
 * Function:    f0m_kill()
 *
 * Description: Kill a process being debugger but let the CLD
 *				    available for a next usage
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_kill (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_attach()
 *
 * Description: Find an unsused component-level debugger or start one
 *				if none was found.  Then attach the debugger to the
 *				process 'tp_pid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_attach (int tp_pid);


/* -----------------------------------------------------------------
 * Function:    f0m_detach()
 *
 * Description: Detach a component-level debugger from the process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_detach (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_symbol_file()
 *
 * Description: Load the symbols
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_symbol_file (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    f0m_file()
 *
 * Description: Load a process image into memory
 *				'tid == -1' load a new process image
 *				'tid != -1' reload a process image
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_file (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    f0m_run()
 *
 * Description: Start the execution of a process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_run (int tid, const char *args);


/* -----------------------------------------------------------------
 * Function:    f0m_step()
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps into the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_step (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_next()
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps over the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_next (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_continue()
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_continue (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_finish()
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_finish (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_signal()
 *
 * Description: Send a signal to a process.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_signal (int tid, int signum);


/* -----------------------------------------------------------------
 * Function:    f0m_break()
 *
 * Description: Set a breakpoint.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_break (int tid, const char *file, int line, const char *function);


/* -----------------------------------------------------------------
 * Function:    f0m_delete()
 *
 * Description: Remove a breakpoint.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_delete (int tid, int bpid);


/* -----------------------------------------------------------------
 * Function:    f0m_set_variable()
 *
 * Description: Set 'variable' to 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_set_variable (int tid, const char *variable, const char *expression);


/* -----------------------------------------------------------------
 * Function:	tkof0m_call()
 *
 * Description: Call a function and get it's return value
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_call (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    f0m_evaluate()
 *
 * Description: Evaluate an 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_evaluate (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    f0m_display()
 *
 * Description: Display 'expression' every time the program stops
 *				If 'expression == NULL' list list all expressions/values
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_display (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    f0m_undisplay()
 *
 * Description: Remove a display expression
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_undisplay (int tid, int dispid);


/* -----------------------------------------------------------------
 * Function:    f0m_list()
 *
 * Description: List a source file.  
 *              The arguments 'start_line' and 'function' are exclusive.
 *              The arguments 'n_lines' is optional.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_list (int tid, const char *file, int start_line,
		  const char *function, int n_lines);


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_locals()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_locals (int tid);


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_args()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_args (int tid);

/* -----------------------------------------------------------------
 * Function:    f0m_info_line()
 *
 * Description: Get information about a source line.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_line (int tid, const char *file, int line,
					  const char *function);


/* -----------------------------------------------------------------
 * Function:	tkof0m_info_program()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_program (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_info_stack()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_stack (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_up()
 *
 * Description: Go "up" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_up (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_down()
 *
 * Description: Go "down" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_down (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_frame()
 *
 * Description: Select a specific stack frame
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_frame (int tid, int frame);


/* -----------------------------------------------------------------
 * Function:    f0m_info_break()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_break (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_info_display()
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_display (int tid);


/* -----------------------------------------------------------------
 * Function:    f0m_thread()
 *
 * Description: Switch the current thread.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_thread (int tid, int threadID);


/* -----------------------------------------------------------------
 * Function:    f0m_info_threads()
 *
 * Description: Get information about the existing threads.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_info_threads (int tid);


/* -----------------------------------------------------------------
 * Function:	tkof0m_tty()
 *
 * Description: Set the I/O tty for the process being debugged
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_tty (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    f0m_sendto()
 *
 * Description: Send a command string directly to the debugger.
 *				'tkout' contains the output (NOT PARSED).
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *
f0m_sendto (int tid, const char *command);


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3_send_request ()
 *
 * Description: Send a command already packed in a 'tkin_t' strucure
 *				to process 'tid'.  Does not wait for the reply.
 *
 * Returns:     The request ID
 */
PUBLIC int
f0m_tkin3_send_request (int tid, tkin_t *tk);


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3_get_reply ()
 *
 * Description: Get the reply associated with a certain request
 *				ID. Should use (-1) as the request ID to get any reply.
 *
 * Returns:     The output token
 */
PUBLIC tkout_t *
f0m_tkin3_get_reply (int reqid);


/* -----------------------------------------------------------------
 * Function:    f0m_tkin3()
 *
 * Description: Set a command already packed into a 'tkin_t' strucure
 *				to process 'tid'.
 *
 * Returns:     The 'Output Token'.
 */
#define f0m_tkin(tid,tk) f0m_tkin3((tid),(tk),0)
tkout_t *
f0m_tkin3 (int tid, tkin_t *tk, int delete);


__END_DECLS

#endif /* __F1_LIB_H */
