/* *****************************************************************
 *
 *  $Id: f0s_lib.h,v 1.2 2002/08/06 22:03:34 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/02 16:56:30 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description:
 *
 * -----------------------------------------------------------------
 */
#ifndef __F0_LIB_H
#define __F0_LIB_H


#include "f0s_tkout.h"

 
/* -----------------------------------------------------------------
 * Function:    int status = f0s_initialize (void)
 *
 * Description: Initialize all the necessary data structures
 *
 * Returns:     The status
 */
int f0s_initialize (void);


/* -----------------------------------------------------------------
 * Function:    int status = f0s_terminate (void)
 *
 * Description: Terminate the debugging in this node.  Detach from all 
 *				processes and kil all CLDs.
 *
 * Returns:     The status
 */
int f0s_terminate (void);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_tids (const char *options)
 *
 * Description: Get a list of all existing process TIDs.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_tids (const char *options);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_kill (int tid)
 *
 * Description: Kill a process being debugger but let the CLD
 *				    available for a next usage
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_kill (int tid);


/* -----------------------------------------------------------------
 * Function:    f0s_add_debugger ()
 *
 * Description: Add a 'ready to be controlled' debugger to Fiddle
 *				'dbg_pid' should contain the PID of the debugger
 *				'tp_pid' should contain the PID of the target process
 *						 or -1 if process is not loaded or not running yet
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_add_debugger (int dbg_pid, int tp_pid,
						   const char *cmd_channel, const char *rep_channel);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_attach (int tp_pid)
 *
 * Description: Find an unsused component-level debugger or start one
 *				if none was found.  Then attach the debugger to the
 *				process 'tp_pid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_attach (int tp_pid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_detach (int tid)
 *
 * Description: Detach a component-level debugger from the process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_detach (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_symbol_file (int tid, 
 *											 const char *file)
 *
 * Description: Load the symbols
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_symbol_file (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_file (int tid, const char *file)
 *
 * Description: Load a process image into memory
 *				'tid == -1' load a new process image
 *				'tid != -1' reload a process image
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_file (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_run (int tid, const char *args)
 *
 * Description: Start the execution of a process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_run (int tid, const char *args);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_step (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps into the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_step (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_next (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps over the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_next (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_continue (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_continue (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_signal (int tid, int signum)
 *
 * Description: Send a signal to a process.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_signal (int tid, int signum);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_finish (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_finish (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_break (int tid, const char *file,
 *									   int line, const char *function)
 *
 * Description: Set a breakpoint.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_break (int tid, const char *file, int line, const char *function);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_delete (int tid, int bpid)
 *
 * Description: Remove a breakpoint.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_delete (int tid, int bpid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_set_variable (int tid,
 *									   const char *variable,
 *									   const char *expression)
 *
 * Description: Set 'variable' to 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_set_variable (int tid, const char *variable,
						 const char *expression);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f0s_call (int tid,
 *									   const char *expression)
 *
 * Description: Call a function and get it's return value
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_call (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_evaluate (int tid,
 *									   const char *expression)
 *
 * Description: Evaluate an 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_evaluate (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_locals (int tid)
 *
 * Description: Get name and value of local variables 
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_locals (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_args (int tid)
 *
 * Description: Get name and value of function arguments
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_args (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_display (int tid,
 *									     const char *expression)
 *
 * Description: Display 'expression' every time the program stops
 *				If 'expression == NULL' list list all expressions/values
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_display (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_undisplay (int tid, int dispid)
 *
 * Description: Remove a display expression
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_undisplay (int tid, int dispid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_list (int tid, const char *file,
 *									   int start_line,
 *									   const char *function,
 *									   int n_lines)
 *
 * Description: List a source file.  
 *              The arguments 'start_line' and 'function' are exclusive.
 *              The arguments 'n_lines' is optional.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_list (int tid, const char *file, int start_line,
				 const char *function, int n_lines);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_line (int tid, const char *file,
 *									        int line,
 *									        const char *function)
 *
 * Description: Get information about a source line.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_line (int tid, const char *file, int line,
					  const char *function);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f0s_info_program (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_program (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_stack (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_stack (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_up (int tid)
 *
 * Description: Go "up" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_up (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_down (int tid)
 *
 * Description: Go "down" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_down (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_frame (int tid, int frame)
 *
 * Description: Select a specific stack frame
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_frame (int tid, int frame);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_break (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_break (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_display (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_display (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_thread (int tid, int threadID)
 *
 * Description: Switch the current thread.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_thread (int tid, int threadID);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_info_threads (int tid)
 *
 * Description: Get information about the existing threads.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_info_threads (int tid);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f0s_tty (int tid,
 *									  const char *file)
 *
 * Description: Set the I/O tty for the process being debugged
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_tty (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f0s_sendto (int tid, const char *comand)
 *
 * Description: Send a command string directly to the debugger.
 *				'tkout' contains the output (NOT PARSED).
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f0s_sendto (int tid, const char *command);


#endif /* __F0_LIB_H */
