/* *****************************************************************
 *
 *  $Id: f1m_lib.h,v 1.3 2002/08/12 00:29:35 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/08/08 22:00:16 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description: Prototypes of the services provided by FIDDLE-1
 *
 *		f1m_initialize ()		-- Initialize FIDDLE-1
 *		f1m_terminate ()			-- Terminate FIDDLE-1
 *		f1m_tids ()				-- List existing task IDs
 *		f1m_kill ()				-- Kill a target process
 *		f1m_attach ()			-- Attach to a running process
 *		f1m_detach ()			-- Detach from a running process
 *		f1m_symbol_file ()		-- Specify whre to find program symbols
 *		f1m_file ()				-- Load program and symbols into memory
 *		f1m_run ()				-- Run a program
 *		f1m_step ()				-- Executite until the next instruction
 *		f1m_next ()				-- Executite until the next instruction
 *		f1m_continue ()			-- Continue the execution
 *		f1m_finish ()			-- Execute until returning from
 *								   current stack frame
 *		f1m_signal ()			-- Send a signal to a process
 *		f1m_break ()				-- Set a breakpoint
 *		f1m_delete ()			-- Delete a breakpoint
 *		f1m_set_variable ()		-- Change the contents of a variable
 *		f1m_call ()				-- Call a function in the current
 *								   context of the target process
 *		f1m_evaluate ()			-- Evaluate an expression in the current
 *								   context of the target process
 *		f1m_display ()			-- Evaluate and display  an expression 
 *								   every time the process execution stops
 *		f1m_undisplay ()			-- Undisplay an expression
 *		f1m_list ()				-- List the process source code
 *		f1m_info_locals ()		-- Get name and value of local variables
 *		f1m_info_args ()		-- Get name and value of function argument
 *		f1m_info_line ()			-- Get info about current line
 *		f1m_info_program ()		-- Get info about current program
 *		f1m_info_stack ()		-- Get info about current stack frames
 *		f1m_up ()				-- Go up in the stack frame list
 *		f1m_down ()				-- Go down in the stack frame list
 *		f1m_frame ()				-- Select a specific stack frame
 *		f1m_info_break ()		-- Get info about current breakpoint
 *		f1m_info_display ()		-- Get info about current display expressions
 *		f1m_thread ()			-- Select a thread
 *		f1m_info_threads ()		-- Get info about the existing threads
 *		f1m_tty ()				-- Set a TTY for future IO
 *		f1m_sendto ()			-- Send a command directly to the debugger
 *		f1m_tkin ()				-- Send an "encoded" command
 *
 * -----------------------------------------------------------------
 */
#ifndef __F1_LIB_H
#define __F1_LIB_H


#include "glob.h"
#include "f0s_lib.h"
#include "f0m_tkin.h"


__BEGIN_DECLS


#define F1M_SERVER_HOST         "localhost"
#define F1M_SERVER_SERVICE_NUMBER "9899"

/* -----------------------------------------------------------------
 * Function:    int status = f1m_initialize (void)
 *
 * Description: Initialize all the necessary data structures
 *
 * Returns:     The status
 */
int f1m_initialize (void);


/* -----------------------------------------------------------------
 * Function:    int status = f1m_terminate ()
 *
 * Description: Terminate the debugging in this node.  Detach from all 
 *				processes and kil all CLDs.
 *
 * Returns:     The status
 */
#define f1m_terminate() f1m_terminate_1 (0)
int f1m_terminate_1 (int dummy);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_tids (const char *options)
 *
 * Description: Get a list of all existing process TIDs.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_tids (const char *options);


/* -----------------------------------------------------------------
 * Function:    f1m_add_debugger ()
 *
 * Description: Add a 'ready to be controlled' debugger to Fiddle
 *				'dbg_pid' should contain the PID of the debugger
 *				'tp_pid' should contain the PID of the target process
 *						 or -1 if process is not loaded or not running yet
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_add_debugger (const char *machine, int dbg_pid, int tp_pid,
						   const char *cmd_channel, const char *rep_channel);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_kill (int tid)
 *
 * Description: Kill a process being debugger but let the CLD
 *				    available for a next usage
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_kill (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_attach (const char *machine, int tp_pid)
 *
 * Description: Find an unsused component-level debugger or start one
 *				if none was found.  Then attach the debugger to the
 *				process 'tp_pid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_attach (const char *machine, int tp_pid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_detach (int tid)
 *
 * Description: Detach a component-level debugger from the process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_detach (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_symbol_file (int tid, 
 *									        	const char *machine,
 *											    const char *file)
 *
 * Description: Load the symbols
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_symbol_file (int tid, const char *machine, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_file (int tid,
 *										 const char *machine,
 *										 const char *file)
 *
 * Description: Load a process image into memory
 *				'tid == -1' load a new process image
 *				'tid != -1' reload a process image
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_file (int tid, const char *machine, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_run (int tid, const char *args)
 *
 * Description: Start the execution of a process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_run (int tid, const char *args);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_step (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps into the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_step (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_next (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps over the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_next (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_continue (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_continue (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_finish (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_finish (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_signal (int tid, int signum)
 *
 * Description: Send a signal to a process.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_signal (int tid, int signum);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_break (int tid, const char *file,
 *									   int line, const char *function)
 *
 * Description: Set a breakpoint.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_break (int tid, const char *file, int line, const char *function);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_delete (int tid, int bpid)
 *
 * Description: Remove a breakpoint.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_delete (int tid, int bpid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_set_variable (int tid,
 *									   const char *variable,
 *									   const char *expression)
 *
 * Description: Set 'variable' to 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_set_variable (int tid, const char *variable,
						 const char *expression);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f1m_call (int tid,
 *									   const char *expression)
 *
 * Description: Call a function and get it's return value
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_call (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_evaluate (int tid,
 *									   const char *expression)
 *
 * Description: Evaluate an 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_evaluate (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_display (int tid,
 *									     const char *expression)
 *
 * Description: Display 'expression' every time the program stops
 *				If 'expression == NULL' list list all expressions/values
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_display (int tid, const char *expression);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_undisplay (int tid, int dispid)
 *
 * Description: Remove a display expression
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_undisplay (int tid, int dispid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_list (int tid, const char *file,
 *									   int start_line,
 *									   const char *function,
 *									   int n_lines)
 *
 * Description: List a source file.  
 *              The arguments 'start_line' and 'function' are exclusive.
 *              The arguments 'n_lines' is optional.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_list (int tid, const char *file, int start_line,
				 const char *function, int n_lines);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_locals (int tid)
 *
 * Description: Get name and value of local variables
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_locals (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_args (int tid)
 *
 * Description: Get name and value of current function arguments
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_args (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_line (int tid, const char *file,
 *									        int line,
 *									        const char *function)
 *
 * Description: Get information about a source line.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_line (int tid, const char *file, int line,
					  const char *function);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f1m_info_program (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_program (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_stack (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_stack (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_up (int tid)
 *
 * Description: Go "up" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_up (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_down (int tid)
 *
 * Description: Go "down" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_down (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_frame (int tid, int frame)
 *
 * Description: Select a specific stack frame
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_frame (int tid, int frame);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_break (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_break (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_display (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_display (int tid);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_thread (int tid, int threadID)
 *
 * Description: Switch the current thread.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_thread (int tid, int threadID);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_info_threads (int tid)
 *
 * Description: Get information about the existing threads.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_info_threads (int tid);


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f1m_tty (int tid,
 *									  const char *file)
 *
 * Description: Set the I/O tty for the process being debugged
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_tty (int tid, const char *file);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_sendto (int tid, const char *comand)
 *
 * Description: Send a command string directly to the debugger.
 *				'tkout' contains the output (NOT PARSED).
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_sendto (int tid, const char *command);


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f1m_tkin3 (int tid, tkin_t *tk, int delete)
 *
 * Description: Set a command already packed into a 'tkin_t' strucure
 *				to process 'tid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f1m_tkin3 (int tid, tkin_t *tk, int delete);


__END_DECLS

#endif /* __F1_LIB_H */
