/* *****************************************************************
 *
 *  $Id: f2m_back_end.c,v 1.8 2002/09/27 13:31:51 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/09/27 14:24:04 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
 *  Description: Implementation of the back-end of the services
 *				 provided by FIDDLE-1.
 *
 *		f2m_be_initialize ()	-- Initialize FIDDLE-1
 *		f2m_be_terminate ()		-- Terminate FIDDLE-1
 *		f2m_be_is_valid_tid ()	-- Check a given TID is valid
 *
 * -----------------------------------------------------------------
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <netdb.h>

#include <assert.h>
#include <pthread.h>

#include "chain.h"
#include "msg.h"

#include "f0s_tkout.h"

#include "f0m_tkin.h"
#include "f0m_rw_to_file.h"


#include "f1m_lib.h"
#include "f2m_lib.h"
#include "f2m_queues.h"
#include "f2m_back_end.h"


/* #################################################################
 *		#DEFINES, TYPES and GLOBAL (PRIVATE) VARIABLES
 * ################################################################# */
#define NO_TID -1


/* #################################################################
 *		PRIVATE functions
 * ################################################################# */


/* -----------------------------------------------------------------
 * Function:    f2m__tcp_connect ()
 *
 * Description: Connet to a server, in 'host' at address 'serv'
 *				'host' is a name string, a dotted decimal (IPV4)  
 *					   or a hex (IPV6) address string
 *				'serv' is a service name, or a decimal (port number)
 *					   string
 *				(adapted from "Unix Networking Programming", Vol. I, 
 *				 by W. Richard Stevens)
 *
 * Returns:     The socket descriptor
 */
PRIVATE int
f2m__tcp_connect (const char *host, const char *serv)
{
	int sockfd, n;
	struct addrinfo hints, *res, *ressave;

	memset (&hints, 0, sizeof (hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	n = getaddrinfo (host, serv, &hints, &res);
	assert (n == 0);
	for (ressave = res;  res!= NULL;  res = res->ai_next) {
		sockfd = socket (res->ai_family,
						 res->ai_socktype, res->ai_protocol);
		if (sockfd > 0) {
			if (connect (sockfd, res->ai_addr, res->ai_addrlen) == 0)
				break;			/* Success!!! */
			close (sockfd);
		}
		sockfd = -1;
	}
	freeaddrinfo (ressave);
	return sockfd;
}



/* -----------------------------------------------------------------
 * Function:    f2m__be_req_service()
 *
 * Description: Send a service request to FIDDLE main daemon.
 *
 * Returns:     The 'Output Token'.
 */
PRIVATE tkout_t
*f2m__be_req_service (int io_fd, const tkin_t *tkin)
{
	tkout_t *tkout;
	int err;

	/* Send the 'tkin_t' token to the server */
	err = f0m_write_tkin (io_fd, tkin);
	assert (err > 0);
/* 	f0m_tkin_delete (tkin); */

	/* Get the reply */
	tkout = f0m_read_tkout (io_fd);
	assert (tkout != NULL);

	/* Return the reply token */
	return tkout;
}



/* -----------------------------------------------------------------
 * Function:    f2m__getenv_or_default()
 *
 * Description: Get environment variable value, if undef use default 
 *
 * Returns:     []
 */
PRIVATE const char *
f2m__getenv_or_default (const char *env_var, const char *default_value)
{
	const char *val = getenv (env_var);
	if (val == NULL)			/* Environment variable not defined */
		val = default_value;	/* Use default value */
	return val;
}
	



/* -----------------------------------------------------------------
 * Function:    f2m__launch_server()
 *
 * Description: Check wich remote shell command to use (e.g., rsh,
 *				ssh), fork() tolaunch the 'f1m_server'
 *
 * Returns:     []
 */
PRIVATE int
f2m__launch_server (const char *f1mserv_mach, const char *f1mserv_service, 
					int flags, const char *argv[], char server[])
{
	int i;
	int f1mserv_fd;
	const char *rsh, *server_dir, *xdr;;

	/* Coudn't connect */
#ifdef EBUG
	msg_pid_write ("Couldn't connect to FIDDLE-1m server in %s:%s\n",
				   f1mserv_mach, f1mserv_service);
#endif
	if (!(flags & START_SERVER)) {
		msg_pid_fatal ("Are you sure the server running?\n\n");
		exit (1);			/* Should never be reached */
	}
	
	/* Check if user defined his own RSH command using an evironment
	   variable */
	rsh = f2m__getenv_or_default ("F2M_RSH", RSH_COMMAND);

	xdr = f2m__getenv_or_default ("FIDDLE_XDR", "jml");
	assert (xdr != NULL);
	
#ifdef EBUG
	msg_pid_write ("Starting a new FIDDLE-1m server in %s:%s\n\n",
				   f1mserv_mach, f1mserv_service);
#endif
	server_dir = f2m__getenv_or_default ("F1M_SERVER_DIR", SERVER_DIR);
	sprintf (server, "%s/%s", server_dir, F1M_SERVER);
	argv[0] = rsh;
	argv[1] = f1mserv_mach;
	argv[2] = server;
	argv[3] = "-x";
	argv[4] = xdr;
	argv[5] = (flags & KEEP_SERVER) ? "-k" : NULL;
	argv[6] = NULL;
#ifdef EBUG
	printf ("Executing:");
	for (i = 0;  argv[i] != NULL; i++)
		printf (" %s", argv[i]);
	puts("");
#endif

	/* Fork lo launch the server */
	if (fork () == 0) {
		close (0);
		close (1);
		execvp ((char *)argv[0], (char **)argv);
		assert (0);
	}
	
	/* Wait for f1mserver to start for, at most, 1e5*50 usec = 5 sec */
#define N_TRIES		50
#define WAIT_TIME	1e5
	for (i = 0;  i < N_TRIES;  i++) {
		f1mserv_fd = f2m__tcp_connect (f1mserv_mach, f1mserv_service);
		if (f1mserv_fd >= 0) break;	/* Success */
		usleep (WAIT_TIME);
	}
	return(f1mserv_fd);
}

PRIVATE void
f2m__register_client(int f1mserv_fd)
{
	tkin_t *tkin;
	tkout_t *tkout;
#define CLI_BUFSIZE 128
	char cli_filename[CLI_BUFSIZE];
	int cli_pid;
	int cli_filenamesize;
	char *p_cli_filename;

	cli_pid = getpid ();
#if defined linux
	cli_filenamesize = readlink ("/proc/self/exe", cli_filename,
								 CLI_BUFSIZE);
#else
#error ***** I do not know how to obtain my own filename in this OS *****
#endif
	assert (cli_filenamesize > 0
			&&  cli_filenamesize < CLI_BUFSIZE-1);
	cli_filename[cli_filenamesize] = '\0';
	p_cli_filename = strrchr (cli_filename, '/');
	if (p_cli_filename == NULL)
		p_cli_filename = cli_filename;
	else
		++p_cli_filename;
	tkin = f0m_tkin_new (NO_TID, C_REGISTER_CLIENT,
						 cli_pid, p_cli_filename);
	tkout = f2m__be_req_service (f1mserv_fd, tkin); /* Register client */
	assert (tkout != NULL);
}


/* -----------------------------------------------------------------
 * Function:    f2m__find_or_start_f1mserver()
 *
 * Description: Connect to FIDDLE main daemon, and initialize the
 *				communications (receive/send) channel
 *
 * Returns:     The I/O descriptor
 */
PRIVATE int
f2m__find_or_start_f1mserver (const char *f1mserv_mach,
							  const char *f1mserv_service, int flags)
{
	const char *argv[10];
	char server[256];
	int i;
	int f1mserv_fd;					/* The I/O channel */
	const char *rsh;
	const char *server_dir;
	struct hostent *he;

	/* If omitted, fill variables with default values */
	if (f1mserv_mach == NULL  ||  *f1mserv_mach == '\0')
		f1mserv_mach = f2m__getenv_or_default ("F1M_SERVER_HOST",
											   F1M_SERVER_HOST);
	if (f1mserv_service == NULL  ||  *f1mserv_mach == '\0')
		f1mserv_service = f2m__getenv_or_default ("F1M_SERVER_SERVICE_NUMBER",
												  F1M_SERVER_SERVICE_NUMBER);

	/* Use the official host name, not aliases */
	he = gethostbyname (f1mserv_mach);
	assert (he != NULL);
	assert (he->h_name != NULL);
	f1mserv_mach = he->h_name;

	/* Connet to the server */
	f1mserv_fd = f2m__tcp_connect (f1mserv_mach, f1mserv_service);

	if (f1mserv_fd < 0)			/* Failed.  Try to start a new server */
		f1mserv_fd = f2m__launch_server (f1mserv_mach, f1mserv_service,
										 flags, argv, server);

	if (f1mserv_fd >= 0) 	/* Success.  Register client. */
		f2m__register_client (f1mserv_fd);
	else {					/* Failure. Still couldn't find server */
		msg_pid_fatal ("Couldn't connect to FIDDLE main daemon in %s/%s\n\tThis means serious problems!!!  :-(\n\n\n", f1mserv_mach, f1mserv_service);
	}
	
	return f1mserv_fd;
}



/* -----------------------------------------------------------------
 * Function:    f2m__do_service()
 *
 * Description: Executed in a thread.  Execute the service request and
 *				enqueue the reply.
 *
 * Returns:     NULL
 */
PRIVATE void *
f2m__do_service(void *_req)
{
	int i;
	tkout_t *tko;
	request_t *req = (request_t *)_req;

	assert (req != NULL);
	tko = f2m__be_req_service (req->fd, req->tk); /* Execute service */
	assert (tko);
	f2m_enqueue_reply (req->id, tko);

	/* Cleanup memory */
	free (req);
	return NULL;
}




/* -----------------------------------------------------------------
 * Function:    f2m__service_thread()
 *
 * Description: Main loop for the service thread
 *
 * Returns:     NULL
 */
PRIVATE void *
f2m__service_thread (void *arg)
{
	int io_fd = (int)arg;
	for (;;) {
		tkout_t   *tko;
		int status;

		/* Wait for a request on queue */
		request_t *req = f2m_dequeue_request ();
		assert (req != NULL);
		req->fd = io_fd;
		
		 /* Execute service */
		status = pthread_create (&(req->thid), NULL, f2m__do_service,
								 (void *)req);
		assert (status == 0);
		pthread_detach (req->thid);
	}
	assert (0);					/* This point should never be reached */
}




/* #################################################################
 *		PUBLIC functions
 * ################################################################# */
 
/* -----------------------------------------------------------------
 * Function:    f2m_be_initialize()
 *
 * Description: Start the system thread
 *
 * Returns:     The status
 */
PUBLIC int
f2m_be_initialize (const char *f1mserv_mach, const char *f1mserv_service,
				   int flags)
{
	int status = -1;	
	int main_daemon_fd = f2m__find_or_start_f1mserver (f1mserv_mach,
													   f1mserv_service, flags);
	
	if (main_daemon_fd >= 0) {
		pthread_t *thid = malloc (sizeof (*thid));
		assert (thid != NULL);
		/* Start a new thread to process all the requests */
		status = pthread_create (thid, pthread_attr_default,
								 f2m__service_thread, (void *)main_daemon_fd);
	}
	return status;
}


/* -----------------------------------------------------------------
 * Function:    f2m_be_terminate()
 *
 * Description: Terminate the system thread
 *
 * Returns:     The status
 */
PUBLIC int
f2m_be_terminate (int halt)
{
	/* Terminate threads */
	/* NOT DONE */
	return 0;
}

