/* *****************************************************************
 *
 *  $Id: f2m_lib.c,v 1.4 2002/09/27 12:43:03 jml Exp $
 *
 *  Copyright (c) 1999 by Dep.Informatica of the FCT/UNL
 *  Time-stamp: <02/09/26 13:27:21 jml>
 *
 *  Joao Lourenco <Joao.Lourenco@di.fct.unl.pt>
 *
 *  Parallel and Distributed Processing Group
 *  Departamento de Informatica
 *  Faculdade de Ciencias e Tecnologia
 *  Universidade Nova de Lisboa
 *
 * -----------------------------------------------------------------
 *
  *  Description: Implementation of the services provided by FIDDLE-2
 *
 *		f2m_initialize ()		-- Initialize FIDDLE-2
 *		f2m_terminate ()		-- Terminate FIDDLE-2
 *		f2m_clients ()			-- List current clients data
 *		f2m_tids ()				-- List existing task IDs
 *		f2m_kill ()				-- Kill a target process
 *		f2m_attach ()			-- Attach to a running process
 *		f2m_detach ()			-- Detach from a running process
 *		f2m_symbol_file ()		-- Specify whre to find program symbols
 *		f2m_file ()				-- Load program and symbols into memory
 *		f2m_run ()				-- Run a program
 *		f2m_step ()				-- Executite until the next instruction
 *		f2m_next ()				-- Executite until the next instruction
 *		f2m_continue ()			-- Continue the execution
 *		f2m_finish ()			-- Execute until returning from
 *								   current stack frame
 *		f2m_signal ()			-- Send a signal to a process
 *		f2m_break ()				-- Set a breakpoint
 *		f2m_delete ()			-- Delete a breakpoint
 *		f2m_set_variable ()		-- Change the contents of a variable
 *		f2m_call ()				-- Call a function in the current
 *								   context of the target process
 *		f2m_evaluate ()			-- Evaluate an expression in the current
 *								   context of the target process
 *		f2m_display ()			-- Evaluate and display  an expression 
 *								   every time the process execution stops
 *		f2m_undisplay ()			-- Undisplay an expression
 *		f2m_list ()				-- List the process source code
 *		f1m_info_locals ()		-- Get name and value of local variables
 *		f1m_info_args ()		-- Get name and value of function argument
 *		f2m_info_line ()			-- Get info about current line
 *		f2m_info_program ()		-- Get info about current program
 *		f2m_info_stack ()		-- Get info about current stack frames
 *		f2m_up ()				-- Go up in the stack frame list
 *		f2m_down ()				-- Go down in the stack frame list
 *		f2m_frame ()				-- Select a specific stack frame
 *		f2m_info_break ()		-- Get info about current breakpoint
 *		f2m_info_display ()		-- Get info about current display expressions
 *		f2m_thread ()			-- Select a thread
 *		f2m_info_threads ()		-- Get info about the existing threads
 *		f2m_tty ()				-- Set a TTY for future IO
 *		f2m_sendto ()			-- Send a command directly to the debugger
 *
 * -----------------------------------------------------------------
 */

#include <stdlib.h>
#include <assert.h>
#include "f0s_tkout.h"
#include "f0s_out_codes.h"

#include "f0m_tkin.h"

#include "f2m_queues.h"
#include "f2m_back_end.h"
#include "f2m_lib.h"
#include "config.h"

/* #################################################################
 *		#DEFINES, TYPES and GLOBAL (PRIVATE) VARIABLES
 * ################################################################# */
#define DELETE_AFTER 1

/* #################################################################
 *		PRIVATE functions
 * ################################################################# */


/* #################################################################
 *		PUBLIC functions
 * ################################################################# */


 
/* -----------------------------------------------------------------
 * Function:    int status = f2m_initialize (const char *host, 
 *											 const char *service,
 *											 int flags)
 *
 * Description: Initialize all the necessary data structures
 *				'host' is the name of the host where the main daemon
 *					   is running
 *				'service' is the service name (or decimal address)
 *				'flags' | START_SERVER if server should be started
 *						| KEEP_SERVER ifserver should be kept alive 
 *
 * Returns:     The status
 */
int f2m_initialize (const char *host, const char *service, int flags)
{
	/* Initialize Queues */
	f2m_queues_init ();
	
	/* Start system thread */
	return f2m_be_initialize (host, service, flags);
}


/* -----------------------------------------------------------------
 * Function:    int status = f2m_terminate (int halt)
 *
 * Description: Terminate the debugging in this node.  Detach from all 
 *				processes and kil all CLDs.
 *
 * Returns:     The status
 */
int f2m_terminate (int halt)
{
	/* Terminate system thread */
	tkin_t *req = f0m_tkin_new (NO_SERVER, C_TERM_SERVER, halt);
	tkout_t *rep = f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
	assert (rep != NULL);
	
	/* Terminate Queues */
	return f2m_queues_terminate (halt);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_clients (const char *options)
 *
 * Description: Get a list of current Fiddle cleints.
 *				'options == "-m machine"' report only clients in <machine>
 *				'options == "-n nmae"' report only clients with <name>
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_clients (const char *options)
{
	tkin_t *req = f0m_tkin_new (NO_SERVER, C_CLIENTS, options);
    return f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_tids (const char *options)
 *
 * Description: Get a list of all existing process TIDs.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_tids (const char *options)
{
	tkin_t *req = f0m_tkin_new (NO_SERVER, C_TIDS, options);
    return f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    f2m_add_debugger ()
 *
 * Description: Add a 'ready to be controlled' debugger to Fiddle
 *				'dbg_pid' should contain the PID of the debugger
 *				'tp_pid' should contain the PID of the target process
 *						 or -1 if process is not loaded or not running yet
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_add_debugger (const char *machine, int dbg_pid, int tp_pid,
						   const char *cmd_channel, const char *rep_channel)
{
	tkin_t *req = f0m_tkin_new (NO_SERVER, C_ADD_DEBUGGER, machine,
								dbg_pid, tp_pid, cmd_channel, rep_channel);
    return f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_kill (int tid)
 *
 * Description: Kill a process being debugger but let the CLD
 *				    available for a next usage
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_kill (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_KILL);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_attach (const char *machine, int tp_pid)
 *
 * Description: Find an unsused component-level debugger or start one
 *				if none was found.  Then attach the debugger to the
 *				process 'tp_pid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_attach (const char *machine, int tp_pid)
{
	tkin_t *req = f0m_tkin_new (NO_SERVER, C_ATTACH, tp_pid, machine);
    return f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_detach (int tid)
 *
 * Description: Detach a component-level debugger from the process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_detach (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_DETACH);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_symbol_file (int tid, 
 *									        	const char *machine,
 *											    const char *file)
 *
 * Description: Load the symbols
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_symbol_file (int tid, const char *machine, const char *file)
{
	tkin_t *req = f0m_tkin_new (tid, C_SYMBOL_FILE, file, machine);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_file (int tid,
 *										 const char *machine,
 *										 const char *file)
 *
 * Description: Load a process image into memory
 *				'tid == -1' load a new process image
 *				'tid != -1' reload a process image
 *				'tid' and 'machine' are exclusive
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_file (int tid, const char *machine, const char *file)
{
	tkin_t *req;

	if (tid == -1)
		tid = NO_SERVER;
	req = f0m_tkin_new (tid, C_FILE, file, machine);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_run (int tid, const char *args)
 *
 * Description: Start the execution of a process
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_run (int tid, const char *args)
{
	tkin_t *req = f0m_tkin_new (tid, C_RUN, args);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_step (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps into the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_step (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_STEP);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_next (int tid)
 *
 * Description: Continue the execution of the process until next
 *					instruction is reached.  Steps over the called
 *					functions.  Blocks the caller until the command is 
 *					 executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_next (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_NEXT);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_continue (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_continue (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_CONTINUE);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_finish (int tid)
 *
 * Description: Continue the execution of the process until a
 *				    breakpoint is reached or the execution finishes.
 *				    Blocks the caller until the command is executed.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_finish (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_FINISH);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_signal (int tid, int signum)
 *
 * Description: Send a signal to a process.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_signal (int tid, int signum)
{
       tkin_t *req;

       req = f0m_tkin_new (tid, C_SIGNAL, tid, signum);
	   return f2m_tkin3 (NO_SERVER, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_break (int tid, const char *file,
 *									   int line, const char *function)
 *
 * Description: Set a breakpoint.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_break (int tid, const char *file, int line, const char *function)
{
	tkin_t *req = f0m_tkin_new (tid, C_BREAK, file, line, function);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_delete (int tid, int bpid)
 *
 * Description: Remove a breakpoint.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_delete (int tid, int bpid)
{
	tkin_t *req = f0m_tkin_new (tid, C_DELETE, bpid);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_set_variable (int tid,
 *									   const char *variable,
 *									   const char *expression)
 *
 * Description: Set 'variable' to 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_set_variable (int tid, const char *variable,
						  const char *expression)
{
	tkin_t *req = f0m_tkin_new (tid, C_SET_VARIABLE, variable, expression);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_evaluate (int tid,
 *									   const char *expression)
 *
 * Description: Evaluate an 'expression'
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_evaluate (int tid, const char *expression)
{
	tkin_t *req = f0m_tkin_new (tid, C_EVALUATE, expression);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_display (int tid,
 *									     const char *expression)
 *
 * Description: Display 'expression' every time the program stops
 *				If 'expression == NULL' list list all expressions/values
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_display (int tid, const char *expression)
{
	tkin_t *req = f0m_tkin_new (tid, C_DISPLAY, expression);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_undisplay (int tid, int dispid)
 *
 * Description: Remove a display expression
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_undisplay (int tid, int dispid)
{
	tkin_t *req = f0m_tkin_new (tid, C_UNDISPLAY, dispid);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f2m_call (int tid,
 *									   const char *expression)
 *
 * Description: Call a function and get it's return value
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_call (int tid, const char *expression)
{
	tkin_t *req = f0m_tkin_new (tid, C_CALL, expression);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_list (int tid, const char *file,
 *									   int start_line,
 *									   const char *function,
 *									   int n_lines)
 *
 * Description: List a source file.  
 *              The arguments 'start_line' and 'function' are exclusive.
 *              The arguments 'n_lines' is optional.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_list (int tid, const char *file, int start_line,
				 const char *function, int n_lines)
{
	tkin_t *req = f0m_tkin_new (tid, C_LIST, file, start_line, function, n_lines);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f2m_tty (int tid,
 *									  const char *file)
 *
 * Description: Set the I/O tty for the process being debugged
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_tty (int tid, const char *file)
{
	tkin_t *req = f0m_tkin_new (tid, C_TTY, file);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_locals (int tid)
 *
 * Description: Get name and value of local variables
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_locals (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_LOCALS);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_args (int tid)
 *
 * Description: Get name and value of current function arguments
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_args (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_ARGS);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_line (int tid, const char *file,
 *									        int line,
 *									        const char *function)
 *
 * Description: Get information about a source line.  
 *              The arguments 'line' and 'function' are exclusive.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_line (int tid, const char *file, int line,
					  const char *function)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_LINE, file, line, function);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:	tkout_t *rep = f2m_info_program (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_program (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_PROGRAM, req);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_stack (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_stack (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_STACK);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_up (int tid)
 *
 * Description: Go "up" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_up (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_UP);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_down (int tid)
 *
 * Description: Go "down" on the stack
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_down (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_DOWN);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_frame (int tid, int frame)
 *
 * Description: Select a specific stack frame
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_frame (int tid, int frame)
{
	tkin_t *req = f0m_tkin_new (tid, C_FRAME, frame);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_break (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_break (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_BREAK);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_display (int tid)
 *
 * Description: Get information about a running program.  
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_display (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_DISPLAY);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_thread (int tid, int threadID)
 *
 * Description: Switch the current thread.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_thread (int tid, int threadID)
{
	tkin_t *req = f0m_tkin_new (tid, C_THREAD, threadID);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_info_threads (int tid)
 *
 * Description: Get information about the existing threads.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_info_threads (int tid)
{
	tkin_t *req = f0m_tkin_new (tid, C_INFO_THREADS);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_sendto (int tid, const char *comand)
 *
 * Description: Send a command string directly to the debugger.
 *				'tkout' contains the output (NOT PARSED).
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_sendto (int tid, const char *command)
{
	tkin_t *req = f0m_tkin_new (tid, C_SENDTO, command);
    return f2m_tkin3 (tid, req, DELETE_AFTER);
}


/* -----------------------------------------------------------------
 * Function:    tkout_t *rep = f2m_tkin3 (int tid, tkin_t *tk, int delete)
 *
 * Description: Set a command already packed into a 'tkin_t' strucure
 *				to process 'tid'.
 *
 * Returns:     The 'Output Token'.
 */
tkout_t *f2m_tkin3 (int tid, tkin_t *tk, int delete)
{
	int reqid;
	tkout_t *rep;

	assert (tk != NULL);
	reqid = f2m_enqueue_request (tk);
	if (reqid >= 0)
		rep = f2m_dequeue_reply (reqid);
	else
		rep = f0s_tkout_new (E_RTS_INVALID_TID);
	if (delete)
		f0m_tkin_delete (tk);
	return rep;
}
